package com.logicbig.example;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.web.WebAppConfiguration;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockHttpServletRequestBuilder;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultHandlers;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;
import org.springframework.test.web.servlet.setup.DefaultMockMvcBuilder;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.web.context.WebApplicationContext;

@RunWith(SpringJUnit4ClassRunner.class)
@WebAppConfiguration
@ContextConfiguration(classes = MyWebConfig.class)
public class ControllerPatchTests {

    @Autowired
    private WebApplicationContext wac;
    private MockMvc mockMvc;

    @Before
    public void setup() {
        DefaultMockMvcBuilder builder = MockMvcBuilders.webAppContextSetup(this.wac);
        this.mockMvc = builder.build();
    }

    @Test
    public void testXmlController() throws Exception {
        long id = 1;
        String content = "new updated content";
        MockHttpServletRequestBuilder builder =
                MockMvcRequestBuilders.patch("/articles/" + id)
                                      .contentType(MediaType.APPLICATION_XML_VALUE)
                                      .accept(MediaType.APPLICATION_XML)
                                      .characterEncoding("UTF-8")
                                      .content(getArticleInXml(1, content));
        this.mockMvc.perform(builder)
                    .andExpect(MockMvcResultMatchers.status()
                                                    .isOk())
                    .andExpect(MockMvcResultMatchers.content()
                                                    .string("Article updated with content: " + content))
                    .andDo(MockMvcResultHandlers.print());
    }

    @Test
    public void testJsonController() throws Exception {
        long id = 1;
        String content = "new updated content";
        MockHttpServletRequestBuilder builder =
                MockMvcRequestBuilders.patch("/articles/" + id)
                                      .contentType(MediaType.APPLICATION_JSON_VALUE)
                                      .accept(MediaType.APPLICATION_JSON)
                                      .characterEncoding("UTF-8")
                                      .content(getArticleInJson(1, content));
        this.mockMvc.perform(builder)
                    .andExpect(MockMvcResultMatchers.status()
                                                    .isOk())
                    .andExpect(MockMvcResultMatchers.content()
                                                    .string("Article updated with content: " + content))
                    .andDo(MockMvcResultHandlers.print());
    }

    @Test
    public void testFormParamController() throws Exception {
        String id = "1";
        String content = "new updated content";
        MockHttpServletRequestBuilder builder =
                MockMvcRequestBuilders.patch("/articles/" + id)
                                      .contentType(MediaType.APPLICATION_FORM_URLENCODED_VALUE)
                                      .accept(MediaType.APPLICATION_FORM_URLENCODED)
                                      .characterEncoding("UTF-8")
                                      .content("id=" + id + "&content=" + content);
        this.mockMvc.perform(builder)
                    .andExpect(MockMvcResultMatchers.status()
                                                    .isOk())
                    .andExpect(MockMvcResultMatchers.content()
                                                    .string("Article updated with content: " + content))
                    .andDo(MockMvcResultHandlers.print());
    }

    private String getArticleInJson(long id, String content) {
        return "{\"id\":\"" + id + "\", \"content\":\"" + content + "\"}";
    }


    private String getArticleInXml(long id, String content) {
        return "<article><id>" + id + "</id><content>" + content + "</content></article>";
    }
}